function printParam( params )
% Display parameter for retrieve, dbStore.
%   printParam( params )
%
%      params( cell, struct ) : parameter
%
% Examples :
%   >> [data params code] = retrieve('SX8O',92000,1,1,0);
%   >> printParam(params);
%   ...
%   >> printParam(params{2});
%
%   >> [time params code] = retrieve_t('SX8O',92000,1,1,1);
%   >> printParam(params);
%
%   >> pl = initParamList();
%   >> pl = setParam(pl,'Shot',98000);
%   >> printParam(pl);
%
% Last update : 2020-7-31
%
% See also
%     getParam setParam retrieve retrieve_t retrieveVersion

if nargin ~= 1 || ~(iscell(params) || ...
        isstruct(params) && all(isfield(params ,{'Pname','Pvalue','Ptype'})) )
   disp('Input argument is short or illegal.');
   disp('Type "help printParam".');
   return;
end
fprintf('Note: Parameter value is shown first 24 characters.\n');
if iscell(params)
    c_num = length(params);
    fprintf('Channel   count =  %d\n',c_num);
    if c_num < 1 
        return
    end
    for c_idx=1:c_num
      print_param(c_idx, params{c_idx});
    end
else
    fprintf('Channel   count =  %d\n',1);
    print_param(1, params);
end
end

function print_param( c_idx, param_list)

  if 1 ~= exist('c_idx','var') | 1 ~= exist('param_list','var')
    fprintf('Undefined input variables.\n');
    fprintf('Usage: print_param( c_idx, param_list)\n');
    return;
  end

  if ~isstruct(param_list)
    fprintf('Input argument is illegal variable.\n');
    return;
  end
  
  n = length(param_list);
  fprintf('\nParameter count =  %d\n',n);
  if 0 == n then
    return;
  end
  
  fprintf('    %24.24s %24.24s %8s\n', 'Name', 'Value', 'Type');
  if and(isfield(param_list,{'Pname','Pvalue','Ptype'}), [1 1 1]) 
    for i = 1:n
      fprintf('%3d %24.24s %24.24s %8s\n', c_idx, ...
        param_list(i).Pname, param_list(i).Pvalue, ...
        mParamType__Name(param_list(i).Ptype) );
    end

  elseif and(isfield(param_list,{'Pname','Pvalue'}), [1 1])
    for i = 1:n
      type_str = class(param_list(i).Pvalue);
      if 'char' == type_str
        val_str = param_list(i).Pvalue;
      elseif 'single' == type_str
        val_str = num2str(param_list(i).Pvalue,7);
      elseif 'double' == type_str
        val_str = num2str(param_list(i).Pvalue,15);
      else  
        val_str = num2str(param_list(i).Pvalue);
      end
      fprintf('%3d %24.24s %24.24s %8s\n', c_idx, ...
        param_list(i).Pname, val_str, type_str );
    end

  else
    fprintf('Input argument is illegal struct.\n');
  end
  fprintf('\n');
end
