#include <octave/oct.h>
#include <octave/int32NDArray.h>
#include "lrt-oct.h"
/*
function [hdl_ptr,code] = LRTopen( diag_name, channel_number, thinnings, ...
    udp_no, packet_size, mhost, verbose )
%
%
%
% loadLibLabcom() is called in LRTstart.
%

% TODO comment out if external func.
%if ~exist('verbose','var')
%    verbose = true;
%end
%
%  DllClass	int	LRTOpen(char *diagname, int chno, LRTINF inf, ...
%                        void *hnd, int udpp, int packetsize);
%  DllClass	int LRTOpen2(char *diagname, int chno, LRTINF inf, ...
%                   void *hnd, int udpp, int packetsize, const char* host);

inf = libstruct('lrtinf');
inf.thinning = thinnings;
hdl_ptr = libpointer('voidPtr',zeros(1,256,'int8'));
if 0 == length(mhost) 
    %[code,diag_name,hdl_ptr]
    lib_func = 'LRTOpen';
    code = calllib( 'librttrans', lib_func, diag_name, channel_number, ...
        inf, hdl_ptr, udp_no, packet_size);
else
    %[code,diag_name,hdl_ptr,mhost]
    lib_func = 'LRTOpen2';
    code = calllib( 'librttrans', lib_func, diag_name, channel_number, ...
        inf, hdl_ptr, udp_no, packet_size, mhost);
end
if code ~= 0
    if verbose
        lrt_message(code, lib_func);
    end
end
end

*/
DEFUN_DLD (LRTopen, args, nargout,
"Open process for real time retrieve-Protocol.\n"
" \n"
"  [hdl_ptr,code] ...\n"
"     = LRTopen( diag_name, channel_number, thinnings, ...\n"
"                    udp_no, packet_size, mhost, verbose )\n"
           )
{
  std::string funcn = "LRTopen ";
  int64_t tmp64v=0;

  std::string diag_name = "";
  unsigned int shot_number;
	unsigned short sub_shot;
  int range_type = -1;
  std::string range_str = "";
  int wait_time_sec=0;
  std::string ret_server= getenv("INDEXSERVERNAME");
  std::string opt_host;
  std::string opt_path;
  bool opt_verbose = false;

  int rd;
  int nargin = args.length();

  for(;;) {
    // check 0.nargin
    rd = -10000;
    if( nargin < 8 || nargin > 9) break;
    // check 0. nargout
    rd--;
    if( nargout != 1 &&  nargout != 2) break;
    // check 1.diag_name
    rd = -10010;
    if (! args(0).is_string())  break;
    diag_name = args(0).string_value();
    if( retrieve_debug ) octave_stdout << funcn <<" diag_name:" << diag_name << "\n";
    // check 2.shot_number
    rd = -10020;
    if ( !args(1).is_scalar_type() || !args(1).is_numeric_type() )  break;
    tmp64v =  args(1).int64_value();
    rd--;
    if( 1 > tmp64v || tmp64v > UINT_MAX )  break;
    shot_number = (unsigned int)tmp64v;
    if( retrieve_debug ) octave_stdout << funcn <<" shot_number:" << shot_number << "\n";
    // check 3.sub_shot
    rd = -10030;
    if ( !args(2).is_scalar_type() || !args(2).is_numeric_type() )  break;
    tmp64v =  args(2).int64_value();
    rd--;
    if( 0 > tmp64v || tmp64v > INT_MAX )  break;
    sub_shot = (unsigned short)tmp64v;
    if( retrieve_debug ) octave_stdout << funcn <<" sub_shot:" << sub_shot << "\n";
    // check 4.host
    rd = -10040;
    if (! args(3).is_string())  break;
    opt_host = args(3).string_value();
    if( retrieve_debug ) octave_stdout << funcn <<" host:" << opt_host << "\n";
    // check 5.path
    rd = -10050;
    if (! args(4).is_string())  break;
    opt_path = args(4).string_value();
    if( retrieve_debug ) octave_stdout << funcn <<" path:" << opt_path << "\n";

    // check 6.wait_time_sec
    rd = -10060;
    if ( !args(5).is_scalar_type() || !args(5).is_numeric_type() )  break;
    tmp64v =  args(5).int64_value();
    rd--;
    if( 0 > tmp64v || tmp64v > INT_MAX )  break;
    wait_time_sec = (int)tmp64v;
    if( retrieve_debug ) octave_stdout << funcn <<" wait_time_sec:" << wait_time_sec << "\n";
    // check 7.range_type 0,1,2,3
    rd = -10070;
    if ( !args(6).is_scalar_type() || !args(6).is_numeric_type() )  break;
    tmp64v =  args(6).int64_value();
    rd--;
    if( 0 > tmp64v || tmp64v > 3 )  break;
    range_type = (int)tmp64v;
    if( retrieve_debug ) octave_stdout << funcn <<" range_type:" << range_type << "\n";
    // check 8.range_str
    rd = -10080;
    if (! args(7).is_string())  break;
    range_str = args(7).string_value();
    if( retrieve_debug ) octave_stdout << funcn <<" range_str:" << range_str << "\n";
    if( nargin > 8 ) {
      // check 9.verbose
      rd = -10090;
#if (OCTAVE_MAJOR_VERSION == 4) && (OCTAVE_MINOR_VERSION == 2)
      if (! args(8).is_bool_type())  break;
#else
      if (! args(8).islogical())  break;
#endif
      opt_verbose = args(8).bool_value();
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";

    rd =0;
    break;
  }
  if( rd == 0 ) {
    IndexNoClosing(1);
    if( 0 != opt_host.length() ) {
      
 	    rd=retrieveOpenDirectWait(diag_name.c_str(), opt_host.c_str(), opt_path.c_str(),
          shot_number, sub_shot, TRANSD_PORT_IGNORE, wait_time_sec);
    }
    else {
 	    rd=retrieveOpenWaitWithRange(diag_name.c_str(), ret_server.c_str(),
          shot_number, sub_shot,
          wait_time_sec,range_type,range_str.c_str());
    }
  }
  octave_value_list ret_vals;
  int32NDArray rda=(dim_vector(1,1));
  rda(0)=rd;
  ret_vals(0) = rda;
  if( 1 < nargout ) ret_vals(1) = ret_server;
  if( 0 > rd ) {
    IndexNoClosing(0);
    if( opt_verbose )  disp_message( rd, funcn);
  }
  return ret_vals;
}
