#include <octave/oct.h>
#include "retrieve-oct.h"

/*
int retrieveGetDTSinfoFromRetrieve_ex(
			int rtr_d, int type, const void* channel, int needDTSlink, int* no_of_channel, 
			unsigned int* ShotNumber, unsigned short* SubShotNumber,
			char* DTSsource, char* DTShostID, char* DTSmoduleID, char* DTStriggerChannel,
			char* CLKsource, char* CLKhostID, char* CLKmoduleID, char* DTSclockChannel,
			int* DTSuserDefine, int64_t* DTStimeArraySize, char* ExtOrInt,
			char* InternalClock, char* SamplingInterval, char* PreSampling,	int64_t* LastCount)
*/
DEFUN_DLD (cRetrieveGetDTSinfoFromRetrieve_ex, args, nargout,
"   [code real_channel shot_number sub_shot ...\n"
"      dts_source, dts_hotsid, dts_moduleid, dts_trig_ch, ...\n"
"      clk_source, clk_hotsid, clk_moduleid, dts_clock_ch, ...\n"
"      dts_user_define, time_array_size, ...\n"
"      int_or_ext, internal_clock, s_sampling_interval, s_pre_samples, ...\n"
"      last_count ] ...\n"
"        = cRetrieveGetDTSinfoFromRetrieve_ex( rtd, channel, need_dts_link, verbose )\n"
" \n"
"      channel : 2,4, ""name"" \n"
           )
{
  const int MIN_IN_ARG = 3;
  const int MAX_IN_ARG = 4;
  const int MAX_OUT_ARG = 19;
  std::string funcn = "cRetrieveGetDTSinfoFromRetrieve_ex";
  
  int rtd = 0;
  int ch_type = 0;
  std::string signal_name = "";
  int channel_no = 0;
  int need_dts_link = 0;
  

  bool opt_verbose = false;
  
  char dts_source[DTS_SOURCE_SIZE]={'\0'};
  char dts_hostid[DTS_HOSTID_SIZE]={'\0'};
  char dts_moduleid[DTS_MODULEID_SIZE]={'\0'};
  char dts_trig_ch[DTS_CH_NAME_SIZE]={'\0'};
  char clk_source[DTS_SOURCE_SIZE]={'\0'};
  char clk_hostid[DTS_HOSTID_SIZE]={'\0'};
  char clk_moduleid[DTS_MODULEID_SIZE]={'\0'};
  char dts_clock_ch[DTS_CH_NAME_SIZE]={'\0'};
  char int_or_ext[RTR_CLK_SOURCE_SIZE]={'\0'};
  char internal_clock[RTR_INTERNAL_CLOCK_SIZE]={'\0'};
  char s_sampling_interval[RTR_SAMP_INTERVAL_SIZE]={'\0'};
  char s_pre_samples[RTR_PRE_SMAPLES_SIZE]={'\0'};
  int64_t last_count = 0;

  int no_of_channel = 0;
  unsigned int shot_number = 0;
  unsigned short sub_shot = 0;
  int dts_user_define = 0;
  int64_t time_array_size = 0;

  int nargin = args.length();
  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
      
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" rtd:" << rtd << "\n";
    // check 2.channel
    if( 0 == (code=get_arg_string( args(1), signal_name )) )  {
      ch_type = 0;
      if( retrieve_debug ) octave_stdout << funcn <<" signal_name:" << signal_name << "\n";
    }
    else if( 0 == (code=get_arg_int( args(1), channel_no )) )  {
      ch_type = 4;
      if( retrieve_debug ) octave_stdout << funcn <<" channel:" << channel_no << "\n";
    }
    else {
      code += OCT_IF_ERR_ARG2;  break;
    }
    // check 3.need_dts_link
    if( 0 != (code=get_arg_int( args(2), need_dts_link )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" need_dts_link:" << need_dts_link << "\n";
    if( nargin > 3 ) {
      // check 4.verbose
      if( 0 != (code=get_arg_bool( args(3), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG4;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )
  
  no_of_channel = channel_no;
  if( 0 == code ) {
    void* channel_p= &channel_no;
    if( 0 == ch_type )  channel_p = (void*)signal_name.c_str();
    code = retrieveGetDTSinfoFromRetrieve_ex( rtd, ch_type, channel_p, need_dts_link,
			        &no_of_channel, &shot_number, &sub_shot,
              dts_source, dts_hostid, dts_moduleid, dts_trig_ch,
              clk_source, clk_hostid, clk_moduleid, dts_clock_ch,
              &dts_user_define, &time_array_size,
              int_or_ext, internal_clock, s_sampling_interval, s_pre_samples, &last_count);
  }
  // retrun process
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 1 < nargout ) ret_vals(1) = no_of_channel;
  if( 2 < nargout ) ret_vals(2) = shot_number;
  if( 3 < nargout ) ret_vals(3) = sub_shot;
//  if( 1 == need_dts_link ) {
    if( 4 < nargout ) ret_vals(4) = dts_source;
    if( 5 < nargout ) ret_vals(5) = dts_hostid;
    if( 6 < nargout ) ret_vals(6) = dts_moduleid;
    if( 7 < nargout ) ret_vals(7) = dts_trig_ch;
    if( 8 < nargout ) ret_vals(8) = clk_source;
    if( 9 < nargout ) ret_vals(9) = clk_hostid;
    if( 10 < nargout ) ret_vals(10) = clk_moduleid;
    if( 11 < nargout ) ret_vals(11) = dts_clock_ch;
    if( 12 < nargout ) ret_vals(12) = dts_user_define;
//  }
  if( 13 < nargout ) ret_vals(13) = time_array_size;
  if( 14 < nargout ) ret_vals(14) = int_or_ext;
  if( 15 < nargout ) ret_vals(15) = internal_clock;
  if( 16 < nargout ) ret_vals(16) = s_sampling_interval;
  if( 17 < nargout ) ret_vals(17) = s_pre_samples;
  if( 18 < nargout ) ret_vals(18) = last_count;
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
    if( 0 > code ) octave_retrieve_close(rtd);    // with IndexNoClosing();
  }
  return(ret_vals);
}