#include <octave/oct.h>
#include "retrieve-oct.h"

/*
RETRIEVE_API int retrieveGetDTSinfox(
		const char *DiagName, unsigned int AShotNumber, short ASubShotNumber,
		short Fastch, short Lastch, const char* IndexServer,
		char* DTSsource, char* DTShostID, char* DTSmoduleID,
		short* DTStriggerChannel, short* DTSclockChannel,
		int* DTSuserDefine, int* DTStimeArraySize,
		char* ClockSource, int* PreSamplings, int* SamplingInterval,
		int* ClkDTSid, char* ClkDTSmodule
		)
*/
DEFUN_DLD (cRetrieveGetDTSinfox, args, nargout,
"   [code dts_source dts_hostid dts_moduleid dts_trig_ch dts_clock_ch ...\n"
"      dts_user_define dts_time_array_size clock_source pre_samplings ...\n"
"      sampling_interval clk_dts_id clk_dts_moduleid] ...\n"
"        = cRetrieveGetDTSinfox( diag_name, shot_number, sub_shot, ...\n"
"                                first_ch, last_ch, index_server, verbose)\n"
           )
{
  const int MIN_IN_ARG = 6;
  const int MAX_IN_ARG = 7;
  const int MAX_OUT_ARG = 13;
  const std::string funcn = "cRetrieveGetDTSinfox";

  std::string diag_name = "";
  unsigned int shot_number = 0;
  unsigned short sub_shot = 0;
  int first_ch = 0;
  int last_ch = 0;
  std::string index_server = "";
  bool opt_verbose = false;

	char dts_source[64]= {'\0'};
  char dts_hostid[64]= {'\0'};
  char dts_moduleid[64]= {'\0'};
	short dts_trig_ch=0, dts_clock_ch=0;
	int   dts_user_define=0, dts_time_array_size=0;
	char clock_source[64]= {'\0'};
  int   pre_samplings=0, sampling_interval=0;
	int   clk_dts_id=0;
  char clk_dts_moduleid[32]= {'\0'};
  
  int nargin = args.length();

  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
      
    // check 1.diag_name
    if( 0 != (code=get_arg_string( args(0), diag_name )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" diag_name:" << diag_name << "\n";
    // check 2.shot_number
    if( 0 != (code=get_arg_uint( args(1), shot_number )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" shot_number:" << shot_number << "\n";
    // check 3.sub_shot
    if( 0 != (code=get_arg_ushort( args(2), sub_shot )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" sub_shot:" << sub_shot << "\n";
    // check 4.first_ch
    if( 0 != (code=get_arg_int( args(3), first_ch )) )  {
      code += OCT_IF_ERR_ARG4;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" first_ch:" << first_ch << "\n";
    // check 5.last_ch
    if( 0 != (code=get_arg_int( args(4), last_ch )) )  {
      code += OCT_IF_ERR_ARG5;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" last_ch:" << last_ch << "\n";
    // check 6.index_server
    if( 0 != (code=get_arg_string( args(5), index_server )) )  {
      code += OCT_IF_ERR_ARG6;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" index_server:" << index_server << "\n";

    if( nargin > 6 ) {
      // check 7.verbose
      if( 0 != (code=get_arg_bool( args(6), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG7;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )

  if( 0 == code ) {
    code = retrieveGetDTSinfox( diag_name.c_str(), shot_number, sub_shot,
            first_ch, last_ch, index_server.c_str(), 
		        dts_source, dts_hostid, dts_moduleid, &dts_trig_ch, &dts_clock_ch,
		        &dts_user_define, &dts_time_array_size, clock_source,
            &pre_samplings, &sampling_interval, &clk_dts_id, clk_dts_moduleid);
  }
  // retrun process
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 1 < nargout ) ret_vals(1) = dts_source;
  if( 2 < nargout ) ret_vals(2) = dts_hostid;
  if( 3 < nargout ) ret_vals(3) = dts_moduleid;
  if( 4 < nargout ) ret_vals(4) = dts_trig_ch;
  if( 5 < nargout ) ret_vals(5) = dts_clock_ch;
  if( 6 < nargout ) ret_vals(6) = dts_user_define;
  if( 7 < nargout ) ret_vals(7) = dts_time_array_size;
  if( 8 < nargout ) ret_vals(8) = clock_source;
  if( 9 < nargout ) ret_vals(9) = pre_samplings;
  if( 10 < nargout ) ret_vals(10) = sampling_interval;
  if( 11 < nargout ) ret_vals(11) = clk_dts_id;
  if( 12 < nargout ) ret_vals(12) = clk_dts_moduleid;

  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
  }

  return(ret_vals);
}
