#include <octave/oct.h>
#include <octave/dNDArray.h>
#include <octave/fNDArray.h>
#include "retrieve-oct.h"

DEFUN_DLD (cRetrieveTimeData, args, nargout,
"Get time data for channel with opened retrieve-Protocol.\n"
" \n"
"   [code data data_length] ...\n"
"      = cRetrieveTimeData( rtd, ch_no, array_size, time_type, verbose)\n"
" \n"
"        time_type = 0:float 1:double \n"
           )
{
  const int MIN_IN_ARG = 3;
  const int MAX_IN_ARG = 5;
  const int MAX_OUT_ARG = 3;
  std::string funcn = "cRetrieveTimeData";

  int rtd = 0;
  int ch_no = 0;
  int64_t array_size =0;
  int time_type = 1;   // 0:float 1:double -1: not support int64_t(pico-sec)
  bool opt_verbose = false;

  int64_t data_length = 0;

  int nargin = args.length();

  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
      
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" rtd:" << rtd << "\n";
    // check 2.ch_no
    if( 0 != (code=get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" ch_no:" << ch_no << "\n";
    // check 3.array_size
    if( 0 != (code=get_arg_int64( args(2), array_size )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" array_size:" << array_size << "\n";

    if( nargin > 3 ) {
      // check 4.time_type
      if( 0 != (code=get_arg_int( args(3), time_type )) )  {
        code += OCT_IF_ERR_ARG4;  break;
      }
      if( 0 != time_type ) time_type = 1;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" time_type:" << time_type << "\n";
    if( nargin > 4 ) {
      // check 5.verbose
      if( 0 != (code=get_arg_bool( args(4), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG5;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )

  octave_value_list ret_vals;
  void*   data  =NULL;
  float*  fdata =NULL;
  double* ddata =NULL;
  for( ;0 == code; ) {
    if( 0 == time_type) { data = fdata = new float[array_size]; }
    else                { data = ddata = new double[array_size];}
    if( NULL == data ) {
      code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;
    }
    
    code = retrieveTimeData(rtd, ch_no,
					data, array_size, &data_length,time_type);
    if( 0 == code ) {
      ret_vals(0) = code;
      if( TIME_FLOAT_TYPE == time_type) {
        FloatNDArray dd(dim_vector(data_length,1));
        for( int64_t x=0;x<data_length;x++ ) { dd(x) = fdata[x];}
        ret_vals(1) = dd;
      }
      else {
        NDArray dd(dim_vector(data_length,1));
        for( int64_t x=0;x<data_length;x++ ) { dd(x) = ddata[x];}
        ret_vals(1) = dd;
      }
    }
    ret_vals(2) = data_length;
    break;
  }  // End of for( ;; ) 
  if( 0 == time_type) {  delete [] fdata;}
  else                {  delete [] ddata;}
  // retrun process
  ret_vals(0) = code;
  if( 0 != code ) {
    if( 1 < nargout ) ret_vals(1) = 0;
    if( 2 < nargout ) ret_vals(2) = 0;
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
    if( 0 > code ) octave_retrieve_close(rtd);    // with IndexNoClosing();
  }

  return(ret_vals);
}
