#include <octave/oct.h>
#include <octave/int32NDArray.h>
#include "retrieve-oct.h"

DEFUN_DLD (cRetrieveTimeInfo, args, nargout,
"Get time information in channel with opened retrieve-Protocol.\n"
" \n"
"   [code dts_source dts_hostid	dts_moduleid dts_trig_ch ...\n"
"         clk_source clk_hostid	clk_moduleid dts_clk_ch ...\n"
"         dts_user_define clock_src	time_array_size ...\n"
"         clock_cycle trigger_delay sampling0timing ] ...n"
"     = cRetrieveTimeInfo( rtd, ch_no, time_type, verbose)\n"
" \n"
"   time_type = 0:float 1:double -1:int64_t(pico-sec)\n"
           )
{
  const int MIN_IN_ARG = 2;
  const int MAX_IN_ARG = 4;
  const int MAX_OUT_ARG = 15;
  const std::string funcn = "cRetrieveTimeInfo";
  
  int rtd = 0;
  int ch_no = 0;
  int time_type = TIME_DOUBLE_TYPE;   // 0:float 1:double -1:int64_t(pico-sec)
  bool opt_verbose = false;

  char dts_source[DTS_SOURCE_SIZE] = {'\0'};
  char dts_hostid[DTS_HOSTID_SIZE] = {'\0'};
  char dts_moduleid[DTS_MODULEID_SIZE] = {'\0'};
  char dts_trig_ch[DTS_CH_NAME_SIZE] = {'\0'};
  char clk_source[DTS_SOURCE_SIZE] = {'\0'};
  char clk_hostid[DTS_HOSTID_SIZE] = {'\0'};
  char clk_moduleid[DTS_MODULEID_SIZE] = {'\0'};
  char dts_clk_ch[DTS_CH_NAME_SIZE] = {'\0'};
	int dts_user_define =0;
  char clock_src[RTR_CLK_SOURCE_SIZE] = {'\0'};
  int64_t time_array_size =0;
  int64_t clock_cycle=0;
  int64_t trigger_delay=0;
  int64_t sampling0timing = 0;

  int nargin = args.length();

  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" rtd:" << rtd << "\n";
    // check 2.ch_no
    if( 0 != (code=get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" ch_no:" << ch_no << "\n";
    if( nargin > 2 ) {
      // check 3.time_type
      if( 0 != (code=get_arg_int( args(2), time_type )) )  {
        code += OCT_IF_ERR_ARG3;  break;
      }
      if     ( TIME_PICO_INT64_TYPE > time_type ) time_type = TIME_PICO_INT64_TYPE;
      else if( TIME_DOUBLE_TYPE  < time_type ) time_type = TIME_DOUBLE_TYPE; 
    }
    if( retrieve_debug ) octave_stdout << funcn <<" time_type:" << time_type << "\n";
    if( nargin > 3 ) {
      // check 4.verbose
      if( 0 != (code=get_arg_bool( args(3), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG4;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )

  if( 0 == code ) {
    code = retrieveTimeInfo(rtd, ch_no,
					dts_source, dts_hostid, dts_moduleid, dts_trig_ch,
					clk_source, clk_hostid, clk_moduleid, dts_clk_ch,
					&dts_user_define, clock_src, &time_array_size,
					&clock_cycle, &trigger_delay, &sampling0timing, time_type);

  }
  // retrun process
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 1 < nargout ) ret_vals(1) = dts_source;
  if( 2 < nargout ) ret_vals(2) = dts_hostid;
  if( 3 < nargout ) ret_vals(3) = dts_moduleid;
  if( 4 < nargout ) ret_vals(4) = dts_trig_ch;
  if( 5 < nargout ) ret_vals(5) = clk_source;
  if( 6 < nargout ) ret_vals(6) = clk_hostid;
  if( 7 < nargout ) ret_vals(7) = clk_moduleid;
  if( 8 < nargout ) ret_vals(8) = dts_clk_ch;
  if( 9 < nargout ) ret_vals(9) = dts_user_define;
  if( 10 < nargout ) ret_vals(10) = clock_src;
  if( 11 < nargout ) ret_vals(11) = time_array_size;
  if( TIME_FLOAT_TYPE == time_type ) {
    if( 12 < nargout ) ret_vals(12) = *((float*)&clock_cycle);
    if( 13 < nargout ) ret_vals(13) = *((float*)&trigger_delay);
    if( 14 < nargout ) ret_vals(14) = *((float*)&sampling0timing);
  }
  else if( TIME_DOUBLE_TYPE == time_type ) {
    if( 12 < nargout ) ret_vals(12) = *((double*)&clock_cycle);
    if( 13 < nargout ) ret_vals(13) = *((double*)&trigger_delay);
    if( 14 < nargout ) ret_vals(14) = *((double*)&sampling0timing);
  }
  else {
    if( 12 < nargout ) ret_vals(12) = clock_cycle;
    if( 13 < nargout ) ret_vals(13) = trigger_delay;
    if( 14 < nargout ) ret_vals(14) = sampling0timing;
  }
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
    if( 0 > code ) octave_retrieve_close(rtd);    // with IndexNoClosing();
  }
  return(ret_vals);
}
