#include <octave/oct.h>
#include <octave/uint32NDArray.h>
#include "dbstore-oct.h"

/*
DBSTORE_API int dbsOpen2(const char *mail_address, const char *diag_name, 
  		 unsigned int shot_number, unsigned short sub_shot, short data_type,
            int cndb_timeout, int cndb_retry);
*/
DEFUN_DLD (dbsOpen, args, nargout,
" Open Process for dbStore. \n"
"   des = dbsOpen( mail_address, data_name, shot_number, sub_shot, ... \n"
"                  data_type, ...) \n"
" \n"
"       mail_address(text) : your mail address \n"
"           When you delete the saved data, this is necessary. \n"
"       data_name(text)    : name of data \n"
"           This uses the data name which you applied for to LABCOM. \n"
"       shot_number(numeric) : SHOT number \n"
"       sub_shot(numeric)    : SHOT sub-number(It is usually 1.) \n"
"       data_type(text)      : 'RAW' or 'ANA' \n"
"           if the data is raw, it is 'RAW', other is 'ANA' \n"
" \n"
"       des : result code \n"
"           0 <  : dbStore's descriptor id, use in other function. \n"
"           0 => : error code. \n"
" \n"
"  OPTIONS : \n"
"       'verbose' : If an error occurs, output messages to display. \n"
"       'cndb_timeout',timeout_sec(numeric) \n"
"                 : seconds of timeout for db connection. \n"
"       'cndb_retry',retry(numeric) \n"
"                 : count of retry for db connection. \n"
" \n"
"  Example : \n"
"       des = dbsOpen('xxx@nifs.ac.jp', 'DATA_X', 1000, 1, 'RAW'); \n"
"       if des > 0  \n"
"          disp('Success open'); \n"
"          .... \n"
"          .... \n"
"          dbsClose(des); \n"
"       end \n"
" \n"
"  Last update : 2018-05-30 \n"
" \n"
"  See also \n"
"      dbsOpen(.oct) dbsWrite(.oct) dbsClose(.oct) dbsAbort(.oct) \n"
"      initParamList(.m) setParam(.m) printParam(.m) \n"
"      dbsOpenFrame(.oct) dbsWriteFrame(.oct) dbsCloseFrame(.oct) \n" 
"      dbsWriteFrames(.oct) dbsMessage(.oct) dbstoreVersion(.oct) dbstore.txt \n"
#ifdef INT64_VALUE_DEFINED
" \n"
" with octave_value.int64_value() is supported.\n"
#endif
           )
{
  const int MIN_IN_ARG = 5;
  const int NUM_OUT_ARG = 1;
  const std::string funcn = "dbsOpen";

  std::string mail_address = "";
  std::string diag_name = "";
  unsigned int shot_number;
	unsigned short sub_shot;
  std::string data_type = "";
  short idata_type =0;
  
  std::string opt_key = "";
  bool opt_verbose = false;
  int cndb_timeout = 5;
  int cndb_retry = 3;
  
  int rd = 0;
  int nargin = args.length();

  int ret_code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG ) { ret_code = OCT_IF_ERR_ARG_NUM_INPUT_ARG; break;}
    // check 0. nargout
    if( nargout != NUM_OUT_ARG ) { ret_code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG; break;}
    
    // check 1.mail_address
    if( 0 != (ret_code=get_arg_string( args(0), mail_address )) ) {
      ret_code += OCT_IF_ERR_ARG1; break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" mail_address:" << mail_address << "\n";
    // check 2.diag_name
    if( 0 != (ret_code=get_arg_string( args(1), diag_name )) ) {
      ret_code += OCT_IF_ERR_ARG2; break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" diag_name:" << diag_name << "\n";
    // check 3.shot_number
    if( 0 != (ret_code=get_arg_uint( args(2), shot_number )) ) {
      ret_code += OCT_IF_ERR_ARG3; break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" shot_number:" << shot_number << "\n";
    // check 4.sub_shot
    if( 0 != (ret_code=get_arg_ushort( args(3), sub_shot )) ) {
      ret_code += OCT_IF_ERR_ARG4; break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" sub_shot:" << sub_shot << "\n";
    // check 5.data_type
    if( 0 != (ret_code=get_arg_string( args(4), data_type )) ) {
      ret_code += OCT_IF_ERR_ARG5; break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" data_type:" << data_type << "\n";
    if( data_type != "ANA" ) {
      idata_type = 2;
    }
    else if ( data_type != "RAW" ) {
      idata_type = 1;
    }
    else {
      ret_code = OCT_IF_ERR_ARG5 + OCT_IF_ERR_ARG_DATA_TYPE;  break;
    }
    break;
  } // End of for( ;; )
  // Option
  int xcode= 0;
  for( int x = MIN_IN_ARG; x < nargin; x++ ) {
    if( 0 == (xcode=get_arg_string( args(x), opt_key )) ) {
      if( opt_key == "verbose" ) {
        opt_verbose = true;
      }
      else if( opt_key == "cndb_timeout" ) {
        x++;
        if( x < nargin ) {
          if( 0 != (xcode=get_arg_int( args(x), cndb_timeout )) ) {
            if( 0 == ret_code )
              ret_code = OCT_IF_ERR_ARG_OPT(x+1) + xcode;
          }
          else {
            if( retrieve_debug ) octave_stdout << funcn <<" cndb_timeout:" << cndb_timeout << "\n";
          }
        }
        else {
          if( 0 == ret_code )
            ret_code = OCT_IF_ERR_ARG_OPT(x+1) + OCT_IF_ERR_ARG_OPTION_NON_VALUE;
        }
      }
      else if( opt_key == "cndb_retry" ) {
        x++;
        if( x < nargin ) {
          if( 0 != (xcode=get_arg_int( args(x), cndb_retry )) ) {
            if( 0 == ret_code )
              ret_code = OCT_IF_ERR_ARG_OPT(x+1) + xcode;
          }
          else {
            if( retrieve_debug ) octave_stdout << funcn <<" cndb_retry:" << cndb_retry << "\n";
          }
        }
        else {
          if( 0 == ret_code )
            ret_code = OCT_IF_ERR_ARG_OPT(x+1) + OCT_IF_ERR_ARG_OPTION_NON_VALUE;
        }
      }
      else {
        if( 0 == ret_code )
          ret_code = OCT_IF_ERR_ARG_OPT(x+1) + OCT_IF_ERR_ARG_NO_SUPPORTED_OPTION;
      }
    }
    else {
      if( 0 == ret_code )
        ret_code = OCT_IF_ERR_ARG_OPT(x+1) + xcode;
    }
  }   // End of option
  
  if( 0 == ret_code ) {
    rd = dbsOpen2(mail_address.c_str(), diag_name.c_str(), 
			 shot_number, sub_shot, idata_type, cndb_timeout, cndb_retry);
  }
  else {
    rd = -ret_code;
  }
  octave_value_list ret_vals;
  int32NDArray rda=(dim_vector(1,1));
  rda(0)=rd;
  ret_vals(0) = rda;
  if( 0 > rd ) {
    if( opt_verbose || retrieve_debug)  {
      if( 0 != ret_code ) octave_dbs_error_message(ret_code, funcn);
      else                octave_dbs_error_message(rd, funcn);
    }
  }
  return ret_vals;
}
