#include <octave/oct.h>
#include "retrieve-oct.h"

DEFUN_DLD (mRetrieveChParam, args, nargout,
"Get Channel parameter with opened retrieve-Protocol.\n"
" \n"
"   [code pname pval ptype] ...\n"
"      = mRetrieveChParam( rtd, ch_no, param_idx, verbose)\n"
           )
{
  const int MIN_IN_ARG = 3;
  const int MAX_IN_ARG = 4;
  const int MAX_OUT_ARG = 4;
  const std::string funcn = "mRetrieveChParam";
  
  int rtd = 0;
  int ch_no = 0;
  int param_idx=0;
  bool opt_verbose = false;
  char pname[PNAME_MAX_LEN] = {'\0'};
  char pval[PVALUE_MAX_LEN] = {'\0'};
  int ptype = 0;

  int nargin = args.length();
  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" rtd:" << rtd << "\n";
    // check 2.ch_no
    if( 0 != (code=get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" ch_no:" << ch_no << "\n";
    // check 3.param_idx
    if( 0 != (code=get_arg_int( args(2), param_idx )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" param_idx:" << param_idx << "\n";
    if( nargin > 3 ) {
      // check 4.verbose
      if( 0 != (code=get_arg_bool( args(3), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG4;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )
  
  if( 0 == code ) {
    code = retrieveChParam(rtd , ch_no , param_idx, pname, pval, &ptype);
  }
  // retrun process
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 1 < nargout ) ret_vals(1) = pname;
  if( 2 < nargout ) ret_vals(2) = pval;
  if( 3 < nargout ) ret_vals(3) = ptype;
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
    octave_retrieve_close(rtd);    // with IndexNoClosing();
  }

  return(ret_vals);
}
