#include <octave/oct.h>
#include <octave/oct-map.h>
#include "retrieve-oct.h"

DEFUN_DLD (mRetrieveChParams, args, nargout,
" Get Channel parameters with opened retrieve-Protocol. \n"
" \n"
"   [code params] ... \n"
"     = mRetrieveChParams( rtd, ch_no, param_num, verbose) \n"
           )
{
  const int MIN_IN_ARG = 3;
  const int MAX_IN_ARG = 4;
  const int MAX_OUT_ARG = 2;
  const std::string funcn = "mRetrieveChParams";

  int rtd = 0;
  int ch_no = 0;
  int param_num=0;
  bool opt_verbose = false;

  int nargin = args.length();
  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" rtd:" << rtd << "\n";
    // check 2.ch_no
    if( 0 != (code=get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" ch_no:" << ch_no << "\n";
    // check 3.param_num
    if( 0 != (code=get_arg_int( args(2), param_num )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" param_num:" << param_num << "\n";
    if( nargin > 3 ) {
      // check 4.verbose
      if( 0 != (code=get_arg_bool( args(3), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG4;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )

  octave_value_list ret_vals;
  
	char** k_tbl=NULL;
	char** v_tbl=NULL;
	int* t_tbl=NULL;
  for( ;0 == code; ) {
		k_tbl=new char *[param_num];
    if( NULL == k_tbl ) { code=OCT_RTR_ERR_MEMORY_ALLOCATE; break;}
		for(int j=0;j<param_num;j++)	k_tbl[j]= NULL;
		v_tbl=new char *[param_num];
    if( NULL == v_tbl ) { code=OCT_RTR_ERR_MEMORY_ALLOCATE; break;}
		for(int j=0;j<param_num;j++)	v_tbl[j]= NULL;
		t_tbl=new int[param_num];
    if( NULL == t_tbl ) { code=OCT_RTR_ERR_MEMORY_ALLOCATE; break;}
		for(int j=0;j<param_num;j++)	{
			k_tbl[j]=new char[PNAME_MAX_LEN];
      if( NULL == k_tbl[j] ) { code=OCT_RTR_ERR_MEMORY_ALLOCATE; break;}
			v_tbl[j]=new char[PVALUE_MAX_LEN];
      if( NULL == v_tbl[j] ) { code=OCT_RTR_ERR_MEMORY_ALLOCATE; break;}
		}
    code = retrieveChParams(rtd , ch_no , k_tbl, v_tbl, t_tbl);
    if( 0 == code && 1 < nargout ) {
      Cell pname_cell(param_num,1);
      Cell pval_cell(param_num,1);
      Cell ptype_cell(param_num,1);
      for(int x=0;x<param_num;x++)	{
        pname_cell(x) = k_tbl[x];
        pval_cell(x)  = v_tbl[x];
        ptype_cell(x) = t_tbl[x];
      }
      octave_map m(dim_vector(param_num,1));
      m.assign("Pname", pname_cell);
      m.assign("Pvalue", pval_cell);
      m.assign("Ptype", ptype_cell);
      ret_vals(1) = m;
    }
    break;
  } // End of for( ;; )
  if( NULL != k_tbl ) {
	  for(int j=0;j<param_num;j++)
      if( NULL != k_tbl[j] ) delete [] k_tbl[j];
	  delete [] k_tbl;
  }
  if( NULL != v_tbl ) {
	  for(int j=0;j<param_num;j++)
      if( NULL != v_tbl[j] ) delete [] v_tbl[j];
	  delete [] v_tbl;
  }
  if( NULL != t_tbl ) delete [] t_tbl;

  // retrun process
  ret_vals(0) = code;
  if( 0 != code ) {
    if( 1 < nargout ) ret_vals(1) = 0;
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
    octave_retrieve_close(rtd);    // with IndexNoClosing();
  }

  return(ret_vals);
}
