#include <octave/oct.h>
#include <octave/int32NDArray.h>
#include "retrieve-oct.h"

DEFUN_DLD (mRetrieveOpen, args, nargout,
"Open process for retrieve-Protocol.\n"
" \n"
"  [rtd, index_server] ...\n"
"     = mRetrieveOpen( diag_name, shot_number, sub_shot, host, path, ...\n"
"                    wait_time_sec, range_type, range_str, verbose)\n"
           )
{
  const int MIN_IN_ARG = 8;
  const int MAX_IN_ARG = 9;
  const int NUM_OUT_ARG1 = 1;
  const int NUM_OUT_ARG2 = 2;
  const std::string funcn = "mRetrieveOpen";
  int64_t tmp64v=0;

  std::string diag_name = "";
  unsigned int shot_number;
	unsigned short sub_shot;
  int range_type = -1;
  std::string range_str = "";
  int wait_time_sec=0;
  std::string ret_server= getenv("INDEXSERVERNAME");
  std::string opt_host;
  std::string opt_path;
  bool opt_verbose = false;

  int rd;
  int nargin = args.length();

  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout != NUM_OUT_ARG1 &&  nargout != NUM_OUT_ARG2) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.diag_name
    if( 0 != (code=get_arg_string( args(0), diag_name )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" diag_name:" << diag_name << "\n";
    // check 2.shot_number
    if( 0 != (code=get_arg_uint( args(1), shot_number )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" shot_number:" << shot_number << "\n";
    // check 3.sub_shot
    if( 0 != (code=get_arg_ushort( args(2), sub_shot )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" sub_shot:" << sub_shot << "\n";
    // check 4.host
    if( 0 != (code=get_arg_string( args(3), opt_host )) )  {
      code += OCT_IF_ERR_ARG4;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" host:" << opt_host << "\n";
    // check 5.path
    if( 0 != (code=get_arg_string( args(4), opt_path )) )  {
      code += OCT_IF_ERR_ARG5;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" path:" << opt_path << "\n";
    // check 6.wait_time_sec
    if( 0 != (code=get_arg_int( args(5), wait_time_sec )) )  {
      code += OCT_IF_ERR_ARG6;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" wait_time_sec:" << wait_time_sec << "\n";
    // check 7.range_type 0,1,2,3
    if( 0 != (code=get_arg_int( args(6), range_type )) )  {
      code += OCT_IF_ERR_ARG7;  break;
    }
    if( 0 > range_type || range_type > 3 )  {
      code = OCT_IF_ERR_ARG7 + OCT_IF_ERR_ARG_INVALID_RANGE_TYPE; break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" range_type:" << range_type << "\n";
    // check 8.range_str
    if( 0 != (code=get_arg_string( args(7), range_str )) )  {
      code += OCT_IF_ERR_ARG8;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" range_str:" << range_str << "\n";
    if( nargin > 8 ) {
      // check 9.verbose
      if( 0 != (code=get_arg_bool( args(8), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG9;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )
  
  if( code == 0 ) {
    IndexNoClosing(1);
    if( 0 != opt_host.length() ) {
 	    rd=retrieveOpenDirectWait(diag_name.c_str(), opt_host.c_str(), opt_path.c_str(),
          shot_number, sub_shot, TRANSD_PORT_IGNORE, wait_time_sec);
    }
    else {
 	    rd=retrieveOpenWaitWithRange(diag_name.c_str(), ret_server.c_str(),
          shot_number, sub_shot,
          wait_time_sec,range_type,range_str.c_str());
    }
  }
  else {
    rd = -code;
  }
  octave_value_list ret_vals;
  int32NDArray rda=(dim_vector(1,1));
  rda(0)=rd;
  ret_vals(0) = rda;
  if( 1 < nargout ) ret_vals(1) = ret_server;
  if( 0 > rd ) {
    IndexNoClosing(0);
    if( opt_verbose ) {
      if( 0 != code ) octave_retrieve_error_message( code, funcn);
      else            octave_retrieve_error_message( rd, funcn);
    }
  }
  return ret_vals;
}
