#include <octave/oct.h>
#include "retrieve-oct.h"

DEFUN_DLD (mRetrieveRetrieveNo, args, nargout,
"Get begin-no and end-no of retrieved data in channel with opened retrieve-Protocol.\n"
" \n"
"   [code begin_no end_no] ...\n"
"      = mRetrieveRetrieveNo( rtd, ch_no, verbose)\n"
           )
{
  const int MIN_IN_ARG = 2;
  const int MAX_IN_ARG = 3;
  const int MAX_OUT_ARG = 3;
  const std::string funcn = "mRetrieveRetrieveNo";
  
  int rtd = 0;
  int ch_no = 0;
  bool opt_verbose = false;
  int64_t begin_no=0;
  int64_t end_no=0;

  int nargin = args.length();

  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" rtd:" << rtd << "\n";
    // check 2.ch_no
    if( 0 != (code=get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" ch_no:" << ch_no << "\n";
    if( nargin > 2 ) {
      // check 3.verbose
      if( 0 != (code=get_arg_bool( args(2), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG3;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )
  
  if( 0 == code ) {
    code = retrieveGetRetrieveNo(rtd, ch_no,&begin_no,&end_no);
  }
  // retrun process
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 1 < nargout ) ret_vals(1) = begin_no;
  if( 2 < nargout ) ret_vals(2) = end_no;
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
  }
  return(ret_vals);
}
