#include <octave/oct.h>
#include "retrieve-oct.h"

DEFUN_DLD (mRetrieveShotInfo, args, nargout,
"Get Shot Information with opened retrieve-Protocol.\n"
" \n"
"   [code last_channel_number syy smm sdd shour smin ssec trans_name] ...\n"
"      = mRetrieveShotInfo( rtd, verbose)\n"
           )
{
  const int MIN_IN_ARG = 1;
  const int MAX_IN_ARG = 2;
  const int MAX_OUT_ARG = 9;
  const std::string funcn = "mRetrieveShotInfo";
  
  int rtd = 0;
  bool opt_verbose = false;
  int64_t tmp64v=0;
  int code = 0;
  unsigned int n_channel=0;
  short syy=0;
  short smm=0;
  short sdd=0;
  short shour=0;
  short smin=0;
  short ssec=0;
  char trans_name[128] = {'\0'};

  int nargin = args.length();

  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" rtd:" << rtd << "\n";
    if( nargin > 1 ) {
      // check 2.verbose
      if( 0 != (code=get_arg_bool( args(1), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG2;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  }
  if( 0 == code ) {
    int comment_size = RTR_COMMENT_SIZE;
    char management[RTR_MANAGE_SIZE];
    char comment[RTR_COMMENT_SIZE];
    code = retrieveShotInfo(rtd , &n_channel ,
							   &syy, &smm, &sdd, &shour,&smin ,&ssec ,
							   management, comment, comment_size, trans_name );
  }
  // retrun process
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 1 < nargout ) ret_vals(1) = n_channel;
  if( 2 < nargout ) ret_vals(2) = syy;
  if( 3 < nargout ) ret_vals(3) = smm;
  if( 4 < nargout ) ret_vals(4) = sdd;
  if( 5 < nargout ) ret_vals(5) = shour;
  if( 6 < nargout ) ret_vals(6) = smin;
  if( 7 < nargout ) ret_vals(7) = ssec;
  if( 8 < nargout ) ret_vals(8) = trans_name;

  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
    octave_retrieve_close(rtd);    // with IndexNoClosing();
  }
  return(ret_vals);
}
